/*
 *	Register map for the MPU6050 available at: https://www.invensense.com/wp-content/uploads/2015/02/MPU-6000-Register-Map1.pdf
 */
#ifndef TINY_MPU6050_H
#define TINY_MPU6050_H

/*
 *	Mandatory includes
 */
#include "Arduino.h"
#include "Wire.h"
#include <string.h>

/*
 *	Default MPU6050 address
 */
#define MPU6050_ADDRESS     	0x68

/*
 *	Configuration registers
 */
#define MPU6050_SMPLRT_DIV  	0x19
#define MPU6050_CONFIG      	0x1A
#define MPU6050_GYRO_CONFIG 	0x1B
#define MPU6050_ACCEL_CONFIG	0x1C
#define MPU6050_PWR_MGMT_1      0x6B

/*
 *	Data registers
 */
#define MPU6050_ACCEL_XOUT_H	0x3B	// Initial register address for accel data
#define MPU6050_GYRO_XOUT_H		0x43	// Initial register address for gyro data

/*
 *  Macros
 */
// For execution
#define ACCEL_TRANSFORMATION_NUMBER     16384
#define GYRO_TRANSFORMATION_NUMBER      65.536
// For digital filter
#define DEFAULT_ACCEL_COEFF             0.02f
#define DEFAULT_GYRO_COEFF              0.98f
// For calibration
#define DISCARDED_MEASURES              100
#define CALIBRATION_MEASURES            5000
#define CHECKING_MEASURES               50
#define ACCEL_PREOFFSET_MAGIC_NUMBER    8
#define GYRO_PREOFFSET_MAGIC_NUMBER     4
#define DEFAULT_ACCEL_DEADZONE          0.002 // m/s²
#define DEFAULT_GYRO_DEADZONE           0.015 // Degrees/second
#define DEADZONE_ATTEMPTS               300

/*
 *	Class
 */
class MPU6050 {

	/*
	 *	Public methods and attributes
	 */
	public:

		// Constructor
		MPU6050 (TwoWire &w);

		// Setup method
		void Initialize ();

        // Method that updates all attributes
        void Execute ();

        // Raw data update methods
        void UpdateRawAccel ();
        void UpdateRawGyro ();

		// Register write method
		void RegisterWrite (byte registerAddress, byte data);

		// Calibrating MPU-6050 method
		void Calibrate (bool console);

		// Gets and sets
        float GetGyroXOffset () { return gyroXOffset; };
        float GetGyroYOffset () { return gyroYOffset; };
        float GetGyroZOffset () { return gyroZOffset; };
        void SetGyroOffsets (float x, float y, float z);

        float GetAccXOffset () { return accXOffset; };
        float GetAccYOffset () { return accYOffset; };
        float GetAccZOffset () { return accZOffset; };
        void SetAccOffsets (float x, float y, float z);

        int16_t GetRawAccX () { return rawAccX; };
        int16_t GetRawAccY () { return rawAccY; };
        int16_t GetRawAccZ () { return rawAccZ; };
        int16_t GetRawGyroX () { return rawGyroX; };
        int16_t GetRawGyroY () { return rawGyroY; };
        int16_t GetRawGyroZ () { return rawGyroZ; };

        float GetAccX () { return accX; };
        float GetAccY () { return accY; };
        float GetAccZ () { return accZ; };
        float GetGyroX () { return gyroX; };
        float GetGyroY () { return gyroY; };
        float GetGyroZ () { return gyroZ; };

        float GetAngAccX () { return angAccX; };
        float GetAngAccY () { return angAccY; };
        float GetAngAccZ () { return angAccZ; };
        float GetAngGyroX () { return angGyroX; };
        float GetAngGyroY () { return angGyroY; };
        float GetAngGyroZ () { return angGyroZ; };

        float GetAngX () { return angX; };
        float GetAngY () { return angY; };
        float GetAngZ () { return angZ; };

        float GetFilterAccCoeff () { return filterAccelCoeff; };
        float GetFilterGyroCoeff () { return filterGyroCoeff; };
        void SetFilterAccCoeff (float coeff);
        void SetFilterGyroCoeff (float coeff);

        float GetAccelDeadzone () { return accelDeadzone; };
        float GetGyroDeadzone () { return gyroDeadzone; };
        void SetAccelDeadzone (float deadzone);
        void SetGyroDeadzone (float deadzone);

	/*
	 *	Private methods and attributes
	 */
	private:

		// I²C stuff
		TwoWire *wire;

		// Gyroscope offsets
		float gyroXOffset, gyroYOffset, gyroZOffset;

        // Accelerometer offsets
        float accXOffset, accYOffset, accZOffset;

		// Raw accel and gyro data
		int16_t rawAccX, rawAccY, rawAccZ, rawGyroX, rawGyroY, rawGyroZ;

		// Readable accel and gyro data
		float accX, accY, accZ, gyroX, gyroY, gyroZ;

		// Integration interval stuff
		long intervalStart;
		float dt;

		// Angle data according to accel and gyro (separately)
		float angGyroX, angGyroY, angGyroZ, angAccX, angAccY, angAccZ;

		// Complememtary filter accel and gyro coefficients
		float filterAccelCoeff, filterGyroCoeff;

		// Angle data w/ complementary filter
		float angX, angY, angZ;

        // Deadzone stuff
        float accelDeadzone, gyroDeadzone, accelDeadzoneThreshold, gyroDeadzoneThreshold;

};

#endif